## $Id: interfaces.py 11589 2014-04-22 07:13:16Z henrik $
##
## Copyright (C) 2014 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
#from datetime import datetime
from zope.component import getUtility
from zope.interface import Attribute, Interface
from zope import schema
from zc.sourcefactory.contextual import BasicContextualSourceFactory
from waeup.ikoba.interfaces import MessageFactory as _
from waeup.ikoba.schema import TextLineChoice, FormattedDate, PhoneNumber
from waeup.ikoba.interfaces import IIkobaObject

class ICustomersUtils(Interface):
    """A collection of methods which are subject to customization.

    """

class ICustomersContainer(IIkobaObject):
    """A customers container contains company customers.

    """
    def addCustomer(customer):
        """Add an ICustomer object and subcontainers.

        """

    def archive(id=None):
        """Create on-dist archive of customers.

        If id is `None`, all customers are archived.

        If id contains a single id string, only the respective
        customers are archived.

        If id contains a list of id strings all of the respective
        customers types are saved to disk.
        """

    def clear(id=None, archive=True):
        """Remove customers of type given by 'id'.

        Optionally archive the customers.

        If id is `None`, all customers are archived.

        If id contains a single id string, only the respective
        customers are archived.

        If id contains a list of id strings all of the respective
        customer types are saved to disk.

        If `archive` is ``False`` none of the archive-handling is done
        and respective customers are simply removed from the
        database.
        """

    unique_customer_id = Attribute("""A unique customer id.""")

class ICustomer(IIkobaObject):
    """Representation of a customer.

    """

